import 'dart:collection';
import 'dart:convert';
import 'dart:io';

import 'package:http/http.dart' as http;
import 'models/static/champion/champion.dart';
import 'models/static/champion/spell.dart';
import 'models/static/item/item.dart';
import 'config.dart';

class StaticApi {
  static final StaticApi api = StaticApi._internal();

  String _version;
  String _versionedUrl;
  String _language;
  String _langUrl;
  http.Client _client;

  factory StaticApi(String version, {String language = 'en_US'}) {
    api._client = http.Client();
    api._version = version;
    api._language = language;
    api._versionedUrl = '${config['cdn']}/$version/data/${api._language}';
    api._langUrl = '${config['cdn']}/languages.json';
    return api;
  }

  StaticApi._internal();

  Future<List<Champion>> getChampions() async {
    var championsList = <Champion>[];
    var response = await _client.get('$_versionedUrl/champion.json');
    var data = parseResponse(response);

    data.forEach((name, champ) {
      if (name != null) {
        championsList.add(Champion.fromJson(champ));
      }
    });

    return championsList;
  }

  Future<List<Champion>> getFullChampions() async {
    var championsList = <Champion>[];
    var response = await _client.get('$_versionedUrl/championFull.json');
    var data = parseResponse(response);

    data.forEach((name, champ) {
      if (name != null) {
        championsList.add(Champion.fromJson(champ));
      }
    });

    return championsList;
  }

  Future<Champion> getChampion(String championName) async {
    var response =
        await _client.get('$_versionedUrl/champion/$championName.json');
    var data = parseResponse(response);
    var champion = Champion.fromJson(data[championName]);

    return champion;
  }

  Future<List<Spell>> getChampionSpells(String championName) async {
    var response =
        await _client.get('$_versionedUrl/champion/$championName.json');
    var data = parseResponse(response);
    var champion = Champion.fromJson(data[championName]);

    return champion.spells;
  }

  Future<List<Item>> getItems() async {
    var itemsList = <Item>[];
    var response = await _client.get('$_versionedUrl/item.json');
    var data = parseResponse(response);

    data.forEach((name, item) {
      if (name != null) {
        itemsList.add(Item.fromJson(item));
      }
    });

    return itemsList;
  }

  Future<List> getLanguages() async {
    var response = await _client.get(_langUrl);
    var data = jsonDecode(response.body);
    return data;
  }

  Future<List> getVersions() async {
    var response = await _client.get('${config['base_url']}/api/versions.json');
    var data = jsonDecode(response.body);
    return data;
  }

  LinkedHashMap<String, dynamic> parseResponse(http.Response response) {
    var parsed = jsonDecode(response.body);
    var data = parsed['data'];

    return data;
  }

  void setLanguage(String language) async {
    var languagesList = await getLanguages();
    if (languagesList.contains(language)) {
      _versionedUrl = '${config['cdn']}/$_version/data/$language';
      stdout.writeln('Change API language to $language');
    } else {
      stderr.writeln('Specified language is not available.');
      stderr.writeln('Available languages: ${languagesList.toString()}');
    }
  }

  void setVersion(String version) async {
    var versionsList = await getVersions();
    if (versionsList.contains(version)) {
      _versionedUrl = '${config['cdn']}/$version/data/$_language';
      stdout.writeln('Change API version to $version');
    } else {
      stderr.writeln('Specified version is not available.');
      stderr.writeln('Available versions: ${versionsList.toString()}');
    }
  }

  String get version {
    return _version;
  }
}
